/*
 * bootloader.h
 * Definitions for boot loader.
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _BOOTLOADER_H_
#define _BOOTLOADER_H_
//#define PROFILE

#include <cslr_intc.h>
#include <cslr_psc.h>

#include <error.h>

#ifdef PROFILE
#include <cslr_tmr.h>
#endif

#include <tistdtypes.h>

#if defined (UBL_USE_NOR_BOOT) && defined (UBL_USE_NAND_BOOT)
#warn "Only NAND or NOR can be selected - Selecting NOR"
#undef UBL_USE_NAND_BOOT
#endif

/* PSC constants */

#define LPSC_TPCC        	4       /*TPCC LPSC*/
#define LPSC_TPTC0       	5	/*TPTC0 LPSC*/
#define LPSC_UHPI		12	/*UHPI LPSC*/
#define LPSC_AEMIF		21	/*AEMIF LPSC*/
#define LPSC_UART0		26 	/*UART0 LPSC*/
#define LPSC_TIMER0		34 	/*TIMER0 LPSC*/
//#define LPSC_ARM		31 	/*ARM LPSC*/
//#define LPSC_GEM		39      /*GEM LPSC*/

#define PD0	0		/*ALWAYS ON POWER DOMAIN*/

/* System module registers */
#define PINMUX0         ((volatile Uint32*)0x01C40000)
#define PINMUX1	        ((volatile Uint32*)0x01C40004)
#define DSPBOOTADDR     ((volatile Uint32*)0x01C40008)
#define INTGEN		((volatile Uint32*)0x01C40010)
#define BOOTCFG 	((volatile Uint32*)0x01C40014)

#define MAGIC_NUMBER_VALID (0xA1ACED00)

#define APP_MAGIC_SAFE		(0xA1ACED00)		/* Safe boot mode */

#define APP_MAGIC_BIN_IMG	(0xA1ACED66)		/* Execute in place supported*/

#define APP_MAGIC_BURN_NOR	(0xA1ACED99)		/* Download via UART & Burn NOR */
#define APP_MAGIC_BURN_NOR_Z	(0xA1ACEDAA)		/* Download via UART & Burn NOR - Image is compressed */
#define APP_MAGIC_BURN_NAND	(0xA1ACEDBB)		/* Download via UART & Burn NAND */
#define APP_MAGIC_BURN_NAND_Z	(0xA1ACEDCC)		/* Download via UART & Burn NAND - Image is compressed */

#define DSP_SEC_ROM_ADDR_0x00E00000 0x00E00000
#define DEVICE_TYPE_MASK_0X00000007 0x00000007
#define BOOTCFG_MASK_0X000000C0 0x000000C0

#define MAGIC_DSP_ID (0xAA55BEEF)

#define UBL_NOR_APP_START_ADDR	(0x02010000)

/* Setting Max Image Size */
#define MAX_IMAGE_SIZE		(0x01000000)
#define RAM_START_ADDR		(0x80000000)
#define RAM_END_ADDR		(0x8FFFFFFF)

typedef struct {
	Uint32 entryPoint;	/* Entry point of the user application */
	Uint32 numPage;		/* Number of pages where boot loader is stored */
	Uint32 block;		/* starting block number where User boot loader is stored */
	Uint32 page;		/* starting page number where boot-loader is stored */
	Uint32 ldAddress;	/* Starting RAM address where image is to copied - XIP Mode */
} NAND_BOOT;

typedef struct {
	Uint32		magicNum;
	Uint32		appSize;
	Uint32		appStartAddr;
	Uint32		ldAddress;	/* Starting RAM address where image is to copied - XIP Mode */
} NOR_BOOT;

#define NOR_BOOT_HDR_SIZE		(16)

typedef enum DeviceType_ {
	TEST = 0,
	EMU,
	SECURE,
	NON_SECURE,
	BAD

} DeviceType;

typedef enum BootMode_ {
	/* Non secure NAND mode */
	NON_SECURE_NAND = 0,
	/* Non secure NOR mode */
	NON_SECURE_NOR,
	/* Non secure UART mode */
	NON_SECURE_UART,
	/* Non secure PCI mode */
	NON_SECURE_PCI,
	/* Non secure UART Programmer */
	NON_SECURE_UART_PRG
} BootMode;

/* Function prototypes */
Uint32 LPSCEnable(Uint8 PdNum, Uint8 LPSCNUM);
void EnableICache();
void DisableICache();
BootMode FindBootMode();
Uint32 NORSecureCopy();
void waitloop (Uint32 count);
Uint32 BootDSP();
Uint32 MagicSwitch(Uint32 magicNum);
Uint32 SFMTDecode(Uint8 *srcAddr, Uint32 size);
void UartProgrammer(void);

#endif //_BOOTLOADER_H_
