/*
 * nand.h
 * Definitions for NAND.
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef _NAND_H_
#define _NAND_H_

#include <tistdtypes.h>
#include <cslr_emif.h>
#include <cslr_edma.h>

/* NAND FLASH ADDRESS */
#define NAND_DATA_ADDR  ((volatile Uint8*)0x42000000)
#define NAND_ALE_ADDR	((volatile Uint8*)0x42040000)
#define NAND_CMD_ADDR	((volatile Uint8*)0x42080000)

#define NAND_TYPE_MASK_0X00000100 (0x00000100)

#define EMIFA_BASE              0x20008000
#define EMIFA_AWCCR             *( volatile Uint32* )( EMIFA_BASE + 0x4 )
#define EMIFA_A1CR              *( volatile Uint32* )( EMIFA_BASE + 0x10 )
#define EMIFA_A2CR              *( volatile Uint32* )( EMIFA_BASE + 0x14 )
#define EMIFA_A3CR              *( volatile Uint32* )( EMIFA_BASE + 0x18 )
#define EMIFA_A4CR              *( volatile Uint32* )( EMIFA_BASE + 0x1c )
#define EMIFA_EIRR              *( volatile Uint32* )( EMIFA_BASE + 0x40 )
#define EMIFA_EIMR              *( volatile Uint32* )( EMIFA_BASE + 0x44 )
#define EMIFA_EIMSR             *( volatile Uint32* )( EMIFA_BASE + 0x48 )
#define EMIFA_EIMCR             *( volatile Uint32* )( EMIFA_BASE + 0x4c )
#define EMIFA_NANDFCR           *( volatile Uint32* )( EMIFA_BASE + 0x60 )
#define EMIFA_NANDFSR           *( volatile Uint32* )( EMIFA_BASE + 0x64 )
#define EMIFA_NANDF1ECC         *( volatile Uint32* )( EMIFA_BASE + 0x70 )
#define EMIFA_NANDF2ECC         *( volatile Uint32* )( EMIFA_BASE + 0x74 )
#define EMIFA_NANDF3ECC         *( volatile Uint32* )( EMIFA_BASE + 0x78 )
#define EMIFA_NANDF4ECC         *( volatile Uint32* )( EMIFA_BASE + 0x7c )

#define NAND_READ_RB                ( EMIFA_NANDFSR & 1 )

/* ------------------------------------------------------------------------ *
 *  Block and Page Organization                                             *
 * ------------------------------------------------------------------------ */
/* Small Page [512/16] */
#define PAGE_ADDR_07_916_1724_2526          1000
#define BLK_ADDR_9_1724_2526                2000

/* Large Page [2048/64] */
#define PAGE_ADDR_07_811_1219_2027          1001
#define BLK_ADDR_1819_2027                  2001


/* NAND FLASH COMMANDS */
#define NAND_LO_PAGE		0
#define NAND_HI_PAGE		1
#define NAND_READ_30H		0x30
#define NAND_EXTRA_PAGE		0x50
#define	NAND_RDID			0x90
#define NAND_RDIDADD		0x00
#define	NAND_RST			0xFF
#define	NAND_PGRM_START	    0x80
#define NAND_PGRM_END	    0x10
#define NAND_RDY			0x40
#define	NAND_PGM_FAIL	    0x01
#define	NAND_BERASEC1	    0x60
#define	NAND_BERASEC2	    0xD0
#define	NAND_STATUS			0x70

/*STATUS OUTPUT*/
#define NAND_STATUS_SUCCESS 0xC0

#define MAX_NAND_SUPPORTED	(33) 			/* Change in accordance with the device info */
#define UNKNOWN_NAND		(0xFF)			/* Unknown device id */
#define NAND_NOT_FOUND          (0)
#define NAND_FOUND              (1)

typedef struct NAND_DEVICE_INFO {
    Uint8 dev_id;               /* DeviceID */
    Uint8 p_cnt;                /* page count per block */
    Uint16 p_byte1;             /* byte count per page(include redundant parts)*/
    Uint8 blk_shift;		/* Number of bits by which block address is to be shifted */
    Uint8 addr_cycles;		/* Number of address cycles */
} NAND_DEVICE_INFO;

/***    NAND_info structure */
typedef struct  {
	Uint8 nand_width;	/* NAND width 1->16 bits 0->8 bits */
    Uint8 dev_code;             /* NAND_DevTable index */
    Uint8 page_cnt;             /* page count per block */
    Uint8 addr_cycles;		/* Number of address cycles */
    Uint8 big_block;		/* 1 - Big block device, 0 - small block device */
    Uint16 page_size;		/* Number of bytes in a page including extra data 264/528 */
    Uint8 blk_shift;		/* Number of bits by which block address is to be shifted */
    Uint8 page_shift;		/* Number of bits by which page address is to be shifted */
} NAND_INFO;

/* Prototypes of functions for NAND flash */
void NANDConfig();
Uint32 NANDInit();
Uint32 NANDGetDetails();
Uint32 NANDReadPage(Uint32 start_page, Uint32 *rx, Uint32 page_count);
Uint32 NANDWaitRdy();
Int16 _NAND_busywait( Int32 timeout );
Uint32 NANDWaitRdyReg();
Uint32 NANDSpareECCRead (Uint8* readPtr);
void NANDReadNBytes(Uint32 numBytes, Uint8* destAddr);
Uint32 NANDReadNBytesDMA(Uint32 numBytes, Uint8* destAddr);

inline void NANDECCStart (void) {
	  CSL_EMIF_0_REGS->NANDFCR |= (1<<8);
}

inline Uint32 NANDECCRead (void) {
	 return CSL_EMIF_0_REGS->NANDF1ECC;
}

/* copies the UBL from NAND to IRAM */
Uint32 NANDCopy();

/* copies the UBL from NAND to IRAM and decrypt it(SECURE mode)*/
Uint32 NANDSecureCopy();

#endif //_NAND_H_
