/*
 * nor.c
 *
 * Copyright (C) 2009 Texas Instruments Incorporated - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/** \file   nor.c
    \brief  DaVinci Secondary boot-loader, NOR driver file

    NOTE: THIS FILE IS PROVIDED FOR INITIAL DEMO RELEASE AND MAY BE
          REMOVED AFTER THE DEMO OR THE CONTENTS OF THIS FILE ARE SUBJECT
          TO CHANGE.

    @author    Amit Chatterjee
    @version    0.1 - Created
**/

#include <bootloader.h>

#ifdef UBL_USE_NOR_BOOT

#include <nor.h>

#define USE_NON_BUFFER_MODE

typedef volatile unsigned int		VUint32;
typedef volatile unsigned short 	VUint16;
typedef volatile unsigned char	 	VUint8;
//
// Generic masks.
//
#define BIT0    0x00000001
#define BIT1    0x00000002
#define BIT2    0x00000004
#define BIT3    0x00000008
#define BIT4    0x00000010
#define BIT5    0x00000020
#define BIT6    0x00000040
#define BIT7    0x00000080
#define BIT8    0x00000100
#define BIT9    0x00000200
#define BIT10   0x00000400
#define BIT11   0x00000800
#define BIT12   0x00001000
#define BIT13   0x00002000
#define BIT14   0x00004000
#define BIT15   0x00008000
#define BIT16   0x00010000
#define BIT17   0x00020000
#define BIT18   0x00040000
#define BIT19   0x00080000
#define BIT20   0x00100000
#define BIT21   0x00200000
#define BIT22   0x00400000
#define BIT23   0x00800000
#define BIT24   0x01000000
#define BIT25   0x02000000
#define BIT26   0x04000000
#define BIT27   0x08000000
#define BIT28   0x10000000
#define BIT29   0x20000000
#define BIT30   0x40000000
#define BIT31   0x80000000

/* Static Variables */

/* Local Function Definition */
void AMD_Write_Buf_Abort_Reset_Flash( VUint32 plAddress );
Int AMD_Flash_Optimized_Write(VUint32 *plAddress, VUint16 ulData[], VUint32 ullength );
Int AMD_Flash_Write( VUint32 *plAddress, VUint16 ulData );
void AMD_Flash_Erase_All(VUint32 address);
void AMD_Flash_Erase_Block(VUint32 blockaddress);
void AMD_Soft_Reset_Flash(VUint32 addr);

///////////////////////////////////////////////////////////////////////////////
//
// DownloadToNORFlash
//
// Description:
//   Copies RAM Memory region to Flash region
//
// Arguments:
//   VUint32 *startWriteAddress   - Base address of the data to be copied
//   VUint16 *startReadAddress   - Base address of the destination address
//   VUint32  size	            - Number of bytes to copy
//
// Return:
//   Flash Programming Status
//
// Assumptions:
//   Flash addresses are always on a halfword boundary, and the byte count
//   is always even.
//
///////////////////////////////////////////////////////////////////////////////
void  NORWriteNBytes(Uint8 * startWriteAddress,
					 Uint32 size,
					 Uint8 * startReadAddress)
{
	VUint32  flash_addr;
	VUint32  SizeInBytes;
	VUint32  sdram_src;
	VUint16 flash_data;
	VUint16 opt_array[16];   //For buffered writes to specific devices
	VUint32  i;
	Int		result;

	SizeInBytes = size;
	flash_addr = (VUint32) startWriteAddress;
	sdram_src = (VUint32) startReadAddress;

	// Write to the Flash in this loop
	UARTSendData(23, "Writing the NOR Flash\r\n");

	while (SizeInBytes > 0)
  	{
    	if((SizeInBytes < 32) || (flash_addr%32))
		{
			flash_data = *((VUint16 *) sdram_src);
			AMD_Flash_Write((VUint32 *) flash_addr, flash_data);
			SizeInBytes   -= 2;
			flash_addr += 2;
			sdram_src  += 2;
		}
		else
		{
			for(i = 0; i<16; i++)	 // <- *** Copy Data
			{
				opt_array[i] = *((VUint16 *) sdram_src);
				sdram_src += 2;
			}

			result = AMD_Flash_Optimized_Write((VUint32 *)flash_addr, opt_array, i);
			if(result == 0)
			{
				SizeInBytes -= (i*2);
				flash_addr += (i*2);
			}
			else
			{
				for(i = 0; i<16; i++)
				{
					result = AMD_Flash_Write((VUint32 *) flash_addr, opt_array[i]);
					if(result != 0)
					{
						UARTSendData(28,"\r\nNormal write also failed\r\n");
						sdram_src -= 32;
						break;
					}
					flash_addr += 2;
					SizeInBytes -= 2;
				}
			}
		}
		if ((flash_addr & 0xFFFC0000) == flash_addr)
		{
			UARTSendData(31,"NOR Write Succeeded for Size 0x");
			UARTSendInt((flash_addr - (VUint32)startWriteAddress));
			UARTSendData(2,"\r\n");
		}
  	}
}

#define UBL_AEMIF_PINMUX0			*( unsigned int* )0x01C40000
#define UBL_AEMIF_PINMUX1			*( unsigned int* )0x01C40004

#define UBL_AEMIF_WAITCFG			*( unsigned int* )0x01E00004   //Async Wait Cycle Config Register
#define UBL_AEMIF_ACFG2				*( unsigned int* )0x01E00010   //Async Bank2 Config Register
#define UBL_AEMIF_ACFG3				*( unsigned int* )0x01E00014   //Async Bank3 Config Register
#define UBL_AEMIF_ACFG4				*( unsigned int* )0x01E00018   //Async Bank4 Config Register
#define UBL_AEMIF_ACFG5				*( unsigned int* )0x01E0001C   //Async Bank5 Config Register

/* NOR Initialization */
void NORInit ()
{
    // Select ASYNC EMIF Address Lines
    UBL_AEMIF_PINMUX0 = 0xC1F;

    // Program Asynchronous Wait Cycles Configuration Control Register
    UBL_AEMIF_WAITCFG |= 0x0 ;

    // Program Asynchronous Bank3-5 Register - // MAX TIMEOUT - 16-bit
    UBL_AEMIF_ACFG3 = 0x3FFFFFFD;
    UBL_AEMIF_ACFG4 = 0x3FFFFFFD;
    UBL_AEMIF_ACFG5 = 0x3FFFFFFD;

    UBL_AEMIF_ACFG2 = 0
        | ( 0 << 31 ) // selectStrobe      = 0;
        | ( 0 << 30 ) // extWait           = 0;
        | ( 0 << 26 ) // writeSetup        = 0;    //   0 ns
        | ( 3 << 20 ) // writeStrobe       = 3;    //  35 ns
        | ( 0 << 17 ) // writeHold         = 0;    //   0 ns
        | ( 3 << 13 ) // readSetup         = 3;    //  30 ns
        | ( 10<< 7 )  // readStrobe        = 10;   // 120 ns
        | ( 0 << 4 )  // readHold          = 0;    //   0 ns
        | ( 3 << 2 )  // turnAround        = 3;    //  ?? ns ( MAX TIMEOUT )
        | ( 1 << 0 )  // asyncSize         = 1;    // 16-bit bus
        ;

    return;
}

///////////////////////////////////////////////////////////////////////////////
//
// AMD Soft Reset Flash
//
// Description:
//   Resets the flash device internally (i.e. a software reset).
// C Syntax:
//   void AMD_Soft_Reset_Flash(VUint32 addr);
// Arguments:
//   r0: Any valid address in a flash memory bank.
// Notes:
//   Callable from C. Registers r0-r3 are *not* preserved.
//
///////////////////////////////////////////////////////////////////////////////
void AMD_Soft_Reset_Flash(VUint32 addr)
{
	// Reset Flash to be in Read Array Mode
	*((VUint16 *) addr) =  AMD_DEVICE_RESET;
	// return to caller
}

///////////////////////////////////////////////////////////////////////////////
//
// AMD Flash Erase Block
//
// Description:
//   This routine erases the AMD Page specified by the input parameter
//   address. This routine is common for both the AM29LV017B device and
//   the AM29DL800B device.
//
// Arguments:
//   address - address in chip select to be erased
//
// Assumptions:
//   The AM29LV017B device won't become confused when the addresses required
//   by the AM29DL800B are used.
//
///////////////////////////////////////////////////////////////////////////////
void AMD_Flash_Erase_Block(VUint32 blockaddress)
{
	VUint16 *address_cs;
	VUint32 tmp_ptr;

	tmp_ptr  = (VUint32) blockaddress;
	address_cs = (VUint16 *) (tmp_ptr & AMD_ADDRESS_CS_MASK);
	*(address_cs + AMD_CMD0_ADDR) = AMD_ERASE_BLK_CMD0;
	*(address_cs + AMD_CMD1_ADDR) = AMD_ERASE_BLK_CMD1;
	*(address_cs + AMD_CMD2_ADDR) = AMD_ERASE_BLK_CMD2;
	*(address_cs + AMD_CMD0_ADDR) = AMD_ERASE_BLK_CMD3;
	*(address_cs + AMD_CMD1_ADDR) = AMD_ERASE_BLK_CMD4;
	*((VUint16 *) blockaddress) = AMD_ERASE_BLK_CMD5;
	while (*((VUint16 *)blockaddress) != AMD_ERASE_DONE);

	/* Flash Mode: Read Array */
    *address_cs = AMD_DEVICE_RESET;
}

///////////////////////////////////////////////////////////////////////////////
//
// AMD Flash Erase All
//
// Description:
//   This routine erases the entire AMD Chip.
//   This routine is common for both the AM29LV017B device and
//   the AM29DL800B device.
//
// Arguments:
//   address - chip select address  to be erased
//
// Assumptions:
//   The AM29LV017B device won't become confused when the addresses required
//   by the AM29DL800B are used.
//
///////////////////////////////////////////////////////////////////////////////
void AMD_Flash_Erase_All(VUint32 address)
{
	VUint16 *psAddress;
	VUint16 *address_cs;
	VUint32 tmp_ptr;
	VUint16 tmp;

	psAddress = (VUint16 *)address;
	tmp_ptr  = (VUint32) address;
	address_cs = (VUint16 *) (tmp_ptr & AMD_ADDRESS_CS_MASK);
	*(address_cs + AMD_CMD0_ADDR) = AMD_ERASE_CMD0;
	*(address_cs + AMD_CMD1_ADDR) = AMD_ERASE_CMD1;
	*(address_cs + AMD_CMD2_ADDR) = AMD_ERASE_CMD2;
	*(address_cs + AMD_CMD0_ADDR) = AMD_ERASE_CMD3;
	*(address_cs + AMD_CMD1_ADDR) = AMD_ERASE_CMD4;
	*(address_cs + AMD_CMD2_ADDR) = AMD_ERASE_CMD5;

	while (1)
	{
		tmp = *psAddress;
		if(tmp & BIT7)
		{
			break;
		}  else
		{
			if(tmp & BIT5)             // Exceeded Time Limit
			{
				tmp = *psAddress;
				if(tmp & BIT7)
				{
					break;
				}  else
				{
					AMD_Soft_Reset_Flash((VUint32) psAddress);
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//
// AMD Flash Write
//
// Description:
//   This routine writes data to flash memory starting at the specified
//   parameter address.
//
// Arguments:
//   plAddress - address to be erased
//   ulData    - data to be written
///////////////////////////////////////////////////////////////////////////////
Int AMD_Flash_Write( VUint32 *plAddress, VUint16 ulData )
{
	VUint16 *psAddress;
	VUint16 *address_cs;
	VUint16 tmp;
	VUint32 tmp_ptr;

	// Lower WORD.
	psAddress = (VUint16 *)plAddress;
	tmp_ptr  = (VUint32) plAddress;
	address_cs = (VUint16 *) (tmp_ptr & AMD_ADDRESS_CS_MASK);

    *((VUint16 *)address_cs + AMD_CMD0_ADDR) = AMD_PROG_CMD0;
    *((VUint16 *)address_cs + AMD_CMD1_ADDR) = AMD_PROG_CMD1;
    *((VUint16 *)address_cs + AMD_CMD2_ADDR) = AMD_PROG_CMD2;

	*psAddress = ulData;

	// Wait for ready.
	while (1)
	{
		tmp = *psAddress;
		if( (tmp & BIT7)  == (ulData & BIT7))
		{
			break;
		}
		else
		{
			if(tmp & BIT5)             // Exceeded Time Limit
			{
				tmp = *psAddress;
				if( (tmp & BIT7)  == (ulData & BIT7))
				{
					break;
				}
				else
				{
				  	AMD_Soft_Reset_Flash((VUint32) psAddress);
			      	return 1;
				}
			}
		}
	}
    // Return Read Mode
	AMD_Soft_Reset_Flash((VUint32) psAddress);
	// Verify the data.
	if (*psAddress != ulData)
	{
     	return 1;
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// AMD Flash Optimized Write
//
// Description:
//   This routine writes an array of data to flash memory starting beginning
//   at the specified parameter address.
//
// Arguments:
//   plAddress - address to begin writing
//   ulData[]  - data array to be written
//   length    - length of data array
//
// Return:
//   int - 0 Success
//		 - 1 Fail
//
///////////////////////////////////////////////////////////////////////////////
Int AMD_Flash_Optimized_Write(VUint32 *plAddress, VUint16 ulData[], VUint32 ullength )
{
	VUint16 *psAddress;
	VUint16 *address_cs;
	VUint16 *psBlockAddress;
	VUint16 *psPollingAddress;
	VUint16           usPollingData;
	VUint16           tmp;
	VUint32 tmp_ptr;
	VUint16 i;

    // Adjust the timings for Writes
	tmp_ptr  = (VUint32) plAddress;
	psAddress = (VUint16 *)plAddress;
	address_cs = (VUint16 *) (tmp_ptr & AMD_ADDRESS_CS_MASK);
	// Block Address WORD.
	psBlockAddress = (VUint16 *)(tmp_ptr &= 0xFFFF0000);                // 64KB Sectors
	// Write the Write Buffer Load command
    *((VUint16 *)address_cs + AMD_CMD0_ADDR) = AMD_WRT_BUF_LOAD_CMD0;
    *((VUint16 *)address_cs + AMD_CMD1_ADDR) = AMD_WRT_BUF_LOAD_CMD1;
    *((VUint16 *)address_cs + AMD_CMD2_ADDR) = AMD_WRT_BUF_LOAD_CMD2;
	// Write Length
	*psBlockAddress = (VUint16) (ullength-1);
	// Write Data
	for(i=0; i<ullength; i++)
	{
		*psAddress = ulData[i];
        usPollingData    = ulData[i];                                  // store polling data
		psPollingAddress = psAddress;                                  // store polling address
		psAddress++;
	}
    // Program Buffer to Flash Confirm Write
	*psBlockAddress = AMD_WRT_BUF_CONF_CMD0;

	psAddress = psPollingAddress;    // set polling address to last loaded location
	                                 // (minimizes edits to "borrowed" polling routine)s
	// Wait for ready.
	while (1)
	{
		tmp = *psAddress;
		if((tmp & BIT7)  == (usPollingData & BIT7))
		{
			break;
		}  else
		{
			if(tmp & BIT5)             // Exceeded Time Limit
			{
				tmp = *psAddress;
				if( (tmp & BIT7)  == (usPollingData & BIT7))
				{
					break;
				}  else
				{
				  	AMD_Soft_Reset_Flash((VUint32) psAddress);
			      	return 1;
				}
			}
			if(tmp & BIT1)             // Write Buffer Loaded Incorrectly
			{
				tmp = *psAddress;
				if( (tmp & BIT7)  == (usPollingData & BIT7))
				{
					break;
				}  else
				{
				  	AMD_Write_Buf_Abort_Reset_Flash ((VUint32) psAddress);
			      	return 1;
				}
			}
		}
	}
	// Put chip back into read array mode.
	AMD_Soft_Reset_Flash((VUint32) psBlockAddress);
	// Verify Written Data
	psAddress = (VUint16 *)plAddress;
	for(i=0; i<ullength; i++)
	{
	 	if(*psAddress != ulData[i])
        {
			return 1;
        }
		psAddress++;
	}
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//
// AMD Write Buf Abort Reset Flash
//
// Description:
//   Resets the flash device when in a write buffer abort state.
//
// C Syntax:
//   void AMD_Soft_Reset_Flash(VUint32 addr);
//
// Arguments:
//   r0: Any valid address in a flash memory sector.
//
// Notes:
//   Callable from C. Registers r0-r3 are *not* preserved.
//
///////////////////////////////////////////////////////////////////////////////
void AMD_Write_Buf_Abort_Reset_Flash( VUint32 plAddress )
{
	VUint16 *address_cs;
	VUint32 tmp_ptr;

	// Lower WORD.
	tmp_ptr  = (VUint32) plAddress;
	address_cs = (VUint16 *) (tmp_ptr & AMD_ADDRESS_CS_MASK);
	// Reset Flash to be in Read Array Mode
    *((VUint16 *)address_cs + AMD_CMD0_ADDR) = AMD_WRT_BUF_ABORT_RESET_CMD0;
    *((VUint16 *)address_cs + AMD_CMD1_ADDR) = AMD_WRT_BUF_ABORT_RESET_CMD1;
    *((VUint16 *)address_cs + AMD_CMD2_ADDR) = AMD_WRT_BUF_ABORT_RESET_CMD2;
	// return to caller
	return;
}

///////////////////////////////////////////////////////////////////////////////
//
// Erase Flash Block
//
// Description:
//   This routine checks erases flash blocks, if they haven't already been
//   erased, based on a start address, byte count, and flash type.  All counts
//
// Return:
//   status - SUCCESS
///////////////////////////////////////////////////////////////////////////////
Int EraseNORFlash(VUint32 start_address, VUint32 size)
{
	VUint32 addr  = start_address;		//byte address
	VUint32 range = start_address + size;	 	//end address
	VUint32 block_size;
	VUint32 addr_without_cs;

   	while (addr < range)
  	{
	    addr_without_cs = (addr & 0x01FFFFFF);    	// address offset within a CS <32MB

		if (addr_without_cs >= 0x02000000)	// return ERROR
		{
			return(E_FAIL);
		}
		block_size = 0x10000;  // uniform sector size (64 KB)

		// Clear intra-sector byte addresses
		// (round down to first byte address in sector)
		addr &= 0xFFFF0000;

		UARTSendData(16,"Erasing Block 0x");
		UARTSendInt(addr);
		UARTSendData(2,"\r\n");

	    //Increment to the next block
	    AMD_Flash_Erase_Block(addr);
	    addr += block_size;
  	}

  	return(E_PASS);
}

#endif /* UBL_USE_NOR_BOOT */
